//ChopDrop.c
//Version 1.0

//Joe Clussman
//DrQueue@hotmail.com
//No License
//16 OCT 2002

#include <GS4LCD.h>
#include <keypad86.h>


/*GLOBAL VARS*/
char *pSaveSpot = 0x9000;		//haven't made a malloc yet...
signed int OldCX;
signed int OldCY;
signed int OldMX;
signed int OldMY;
signed int OldWX;
signed int OldWY;
unsigned char BladeState = 0;
unsigned char HorseState = 0;
unsigned char ManSquish = 0;
unsigned char UpdateStatusBar = 1;

void drawbox(int X, int Y, unsigned int W, unsigned int H, unsigned char Shade)
{
	
	int DX = X;
	int Xmax = X + W;
	int Ymax = Y + H;
	
	while (Y < Ymax)
	{
		while (DX < Xmax)
			Dot(DX++,Y,Shade);
		DX = X;
		Y++;
	}
}

void ShowStatus(unsigned int score, unsigned char lives)
{
	UpdateStatusBar = 0;
	TextUINT(0,0,score);
	TextUINT(15,0,(unsigned int)lives);
}


//grab background that will be behind OAMs.
void initCopterAndManAndWagon(int CX, int CY, int MX, int MY, int WX, int WY)
{
	OldCX = CX;
	OldCY = CY;
	OldMX = MX;
	OldMY = MY;
	OldWX = WX;
	OldWY = WY;

	GetVidBlock16x8(OldCX         , OldCY, 0, pSaveSpot);
	GetVidBlock16x8(OldCX + 0x0010, OldCY, 1, pSaveSpot);
	GetVidBlock16x8(OldCX + 0xFFFC, OldCY + 0xFFF8, 5, pSaveSpot);
	GetVidBlock16x8(OldCX + 0x000C, OldCY + 0xFFF8, 6, pSaveSpot);
	GetVidBlock16x8(OldMX,          OldMY, 2, pSaveSpot);
	GetVidBlock16x8(OldWX         , OldWY, 3, pSaveSpot);
	GetVidBlock16x8(OldWX + 0x0010, OldWY, 4, pSaveSpot);

	BladeState = 0;
	HorseState = 0;
	ManSquish = 0;
	UpdateStatusBar = 1;
}


//wow this takes tons of space, oh well.
void redrawCMWS(int CX, int CY, int MX, int MY, int WX, int WY, unsigned int score, unsigned char lives)
{
	PutVidBlock16x8(OldCX + 0xFFFC, OldCY + 0xFFF8, 5, pSaveSpot);
	PutVidBlock16x8(OldCX + 0x000C, OldCY + 0xFFF8, 6, pSaveSpot);
	PutVidBlock16x8(OldCX         , OldCY, 0, pSaveSpot);
	PutVidBlock16x8(OldCX + 0x0010, OldCY, 1, pSaveSpot);
	PutVidBlock16x8(OldMX         , OldMY, 2, pSaveSpot);
	PutVidBlock16x8(OldWX         , OldWY, 3, pSaveSpot);
	PutVidBlock16x8(OldWX + 0x0010, OldWY, 4, pSaveSpot);

	//ARGH
	if (UpdateStatusBar != 0)
		ShowStatus(score, lives);

	GetVidBlock16x8(CX + 0xFFFC, CY + 0xFFF8, 5, pSaveSpot);
	GetVidBlock16x8(CX + 0x000C, CY + 0xFFF8, 6, pSaveSpot);
	GetVidBlock16x8(CX         , CY, 0, pSaveSpot);
	GetVidBlock16x8(CX + 0x0010, CY, 1, pSaveSpot);
	GetVidBlock16x8(MX         , MY, 2, pSaveSpot);
	GetVidBlock16x8(WX         , WY, 3, pSaveSpot);
	GetVidBlock16x8(WX + 0x0010, WY, 4, pSaveSpot);

	OAM8x8(CX       , CY, 1);
	OAM8x8(CX+0x0008, CY, 2);
	OAM8x8(CX+0x0010, CY, 3);
	OAM8x8(MX       , MY, 8  + ManSquish);
	OAM8x8(WX       , WY, 11);
	OAM8x8(WX+0x0008, WY, 12);
	OAM8x8(WX+0x0010, WY, 13 + HorseState);

	if (BladeState == 0)
	{
		OAM8x8(CX + 0xFFFC, CY + 0xFFF8, 4);
		OAM8x8(CX + 0x0004, CY + 0xFFF8, 5);
		BladeState = 1;
	}
	else
	{
		OAM8x8(CX + 0x0004, CY + 0xFFF8, 6);
		OAM8x8(CX + 0x000C, CY + 0xFFF8, 7);
		BladeState = 0;
	}

	OldMX = MX;
	OldMY = MY;
	OldCX = CX;
	OldCY = CY;
	OldWX = WX;
	OldWY = WY;
}


void drawTree(signed int X, signed int Y)
{
	drawbox(X+2, Y,   3,3, 2);
	drawbox(X+1, Y+3, 5,11,2);
	drawbox(X,   Y+5, 7,8, 2);
	drawbox(X+3, Y+14,1,63-(Y+14), 3);
}

void initBackground(void)
{
	signed int col;
	
	ClrScr(0);
	drawbox(0,62,128,2,3);

	drawTree(22,45);
	
	/* a fence? */
	drawbox(0,56,77,2,1);
	drawbox(0,59,77,2,1);
	for(col = 0; col <= 77; col+=11)
		drawbox(col, 55, 1, 7, 1);

	drawTree(97,45);
	drawTree(111,44);
}

unsigned int DoTheDropGame(void)
{
	signed int CX = 60;					// Helicopter coord
	signed int CY = 5;
	signed int MX = 40;					// Stuntman
	signed int MY = 21;
	signed int WX = 3;					// Wagon and horse
	signed int WY = 54;

	unsigned char ManStateVar = 0;		// holding on, falling, etc.
	signed int ManPosInCart = 0;

	unsigned char WagonFineX = 0;		// Wagon Movement
	unsigned char WagonRoundFlag = 0;
	unsigned char WagonVelX = 90;		// Speed of Horse across screen
	unsigned char HorseAnimTimer = 0;

	unsigned int SCORE = 0;
	signed int scoreCalc = 0;
	unsigned int DropHeight = 0;
	signed char dropsLeft = 4;

	//have to formally position OAMs
	initCopterAndManAndWagon(CX, CY, MX, MY, WX, WY);

	do
	{
		/*Helicopter Stuff*/
		if (isKeyPressed(K_DOWN) != 0)
		{
			CY = CY + 0x0001;
			if (CY > 35)
			{
				CY = 35;
			}
		}

		
		if (isKeyPressed(K_LEFT) != 0)
		{
			CX = CX + 0xFFFF;
			if (CX < -4)
			{
				CX = -4;
			}
		}

		if (isKeyPressed(K_RIGHT) != 0)
		{
			CX = CX + 0x0001;
			if (CX > 117)
			{
				CX = 117;
			}
		}

		if (isKeyPressed(K_UP) != 0)
		{
			CY = CY + 0xFFFF;
			if (CY < -7)
			{
				CY = -7;
			}
		}


		/*Wagon Stuff*/
		WagonFineX += WagonVelX;
		while (WagonFineX >= 64)
		{
			//adjust it back down (-64)
			WagonFineX = WagonFineX + 0xC0;

			//did we already round up?
			if (WagonRoundFlag != 0)
			{
				WagonRoundFlag = 0;
			}
			else
			{
				WX = WX + 0x0001;
			}
			
			//animate the horse running across
			if ((HorseAnimTimer++ > 2) && (HorseState != 2))
			{
				HorseAnimTimer = 0;
				if (HorseState == 0)
				{
					HorseState = 1;
				}
				else
				{
					HorseState = 0;
				}
			}
		}
		//want smoother motion (round at half)
		if (WagonFineX > 32)
		{
			WagonRoundFlag = 1;
			WX = WX + 0x0001;
		}

		//has wagon gone past right edge of screen?
		if (WX > 127)
		{
			if (ManStateVar == 3)
				ManStateVar = 0;
			
			HorseState = 0;			//reset wagon & horse
			WX = 0xFFE8;
			WagonVelX += 0xC3;		//Pseudo almost random speed

			if (WagonVelX > 114)
				WagonVelX += 0x8D;

			if (WagonVelX < 33)
				WagonVelX += 33;
		}


		/*poor guy...*/
		switch(ManStateVar)
		{
		case 0:
			MX = CX + 4;	// holding on
			MY = CY + 8;
			ManSquish = 0;	// charmap to use

			//if second is pressed, drop him!
			if (isKeyPressed(K_SECOND) != 0)
			{
				DropHeight = 44 - MY;
				ManStateVar = 1;
			}
			break;
		case 1:
			/* falling */
			if ((MY+6) >= WY)
			{
				ManPosInCart = MX - WX;

				ManStateVar = 4;				//default not in cart
				if (ManPosInCart > 0)
				{
					if (ManPosInCart < 14)
					{
						ManStateVar = 2;
					}
					else
					{
						if (ManPosInCart < 24)	//hit the horsey
							HorseState = 2;
					}
				}
			}			
			MY = MY + 1;
			break;
		case 2:			
			/* on target */
			scoreCalc = ManPosInCart - 7;
			if (scoreCalc < 0)
				scoreCalc = -scoreCalc;
			scoreCalc = 8 - scoreCalc;
			SCORE = SCORE + (scoreCalc * DropHeight);

			scoreCalc = (unsigned int)WagonVelX;
			scoreCalc = (scoreCalc - 32) / 4;
			SCORE = SCORE + scoreCalc;

			UpdateStatusBar = 1;

			ManStateVar = 3;
		case 3:
			/* inside cart */
			MX = WX + ManPosInCart;
			MY = WY + 0xFFFB;
			break;
		
		case 4:
			/* missed cart */
			MY = MY + 1;
			if (MY > 54)
			{
				MY = 54;
				ManStateVar = 5;
			}
			break;
		case 5:
			/* splat */
			ManSquish = 1;
			ManStateVar = 6;
			break;
		case 6:
			ManSquish = 2;
			ManStateVar = 7;
			break;
		case 7:
			ManStateVar = 0;
			UpdateStatusBar = 1;
			dropsLeft--;
		}

		WaitForFrame(8);		//so important for smoothness
								// none of this software delay garbage
		
		redrawCMWS(CX, CY, MX, MY, WX, WY, SCORE, dropsLeft);
	}
	while ( (isKeyPressed(K_EXIT) == 0)&&(dropsLeft > 0) );
	
	return SCORE;
}

void main(void)
{
	unsigned int topScore;
	unsigned int finalScore = 0;
	unsigned char TitleLoopExit = 0;
	unsigned char ExitKeyPressed = 0;

	topScore = LoadHighScore();
	
	do 
	{
		/* title screen */
		ClrScr(0);
		drawbox(0,0,128,16,1);

		TextSTRING(1,0,"Chop Drop      v1.0");
		TextSTRING(3,1,"by Joe Clussman");

		TextSTRING(1,3,"[2nd] = Start");
		TextSTRING(1,4,"[EXIT] = Quit");
		TextSTRING(0,6,"TOP SCORE");
		TextUINT(14,6,topScore);

		
		TitleLoopExit = 0;
		do 
		{
			WaitForFrame(16);

			if (isKeyPressed(K_EXIT) != 0)
			{
				TitleLoopExit = 1;
				ExitKeyPressed = 1;
			}
			
			if (isKeyPressed(K_SECOND) != 0)
			{
				WaitForFrame(20);
				initBackground();
				finalScore = DoTheDropGame();

				TextSTRING(6,3,"Game Over");
				WaitForFrame(120);
				TitleLoopExit = 1;

				if (finalScore > topScore)
				{
					topScore = finalScore;
					TextSTRING(3,4,"New High Score!");
					WaitForFrame(70);
				}
			}
		}
		while (TitleLoopExit == 0);
		
	}
	while (ExitKeyPressed == 0);

	SaveHighScore(topScore);
	ClrScr(0);
}
